@@ -3,11 +3,14 @@ from pathlib import Path
 from typing import Any, Dict, Iterable, List, Union
 
 import mlflow
+import numpy as np
 import pandas as pd
+import shap
 
 from tabml.data_loaders import BaseDataLoader
 from tabml.metrics import BaseMetric, get_instantiated_metric_dict
 from tabml.model_wrappers import BaseModelWrapper
+from tabml.utils import utils
 from tabml.utils.logger import logger
 
 
@@ -43,6 +46,8 @@ class ModelAnalysis:
         need_predict_proba:
             A bool value to tell if method self.model_wrapper.predict_proba is needed.
             Some metrics are computed based on probabilities.
+        training_size: int or None
+            Training sample size for the analysis.
     """
 
     def __init__(
@@ -55,6 +60,7 @@ class ModelAnalysis:
         label_to_analyze: str = "",
         pred_col: str = "prediction",
         pred_proba_col: str = "prediction_probability",
+        training_size: Union[int, None] = None,
     ):
 
         self.data_loader = data_loader
@@ -74,12 +80,47 @@ class ModelAnalysis:
         # True at initialization, then to False right after printing the first overall
         # scores.
         self._show_overall_flag = True
+        self.training_size = training_size
 
     def analyze(self):
-        self._analyze_one_dataset("train")
-        self._analyze_one_dataset("val")
+        self.shap_explain()
+        self._analyze_metrics_one_dataset("train")
+        self._analyze_metrics_one_dataset("val")
 
-    def _analyze_one_dataset(self, dataset_name: str):
+    def shap_explain(self):
+        """Explains model using SHAP https://github.com/slundberg/shap."""
+        # show shap feature importance
+        explainer = shap.Explainer(self.model_wrapper.model)
+        train_feature = self._get_dataset("train")[self.model_wrapper.feature_names]
+        shap_values = explainer(train_feature)
+
+        def _get_shape_values_one_sample(shape_values, index: int):
+            # For LightGBM and XGBoost, shape_values[index].values is a 2d array
+            # representing logit of two classes. They are basically negative of each
+            # other, we only need one.
+            # Related issue https://github.com/slundberg/shap/issues/526.
+            if len(shape_values[index].values.shape) == 2:  # binary classification
+                return shape_values[index].values[:, 0]
+            assert len(shape_values[index].values.shape) == 1, len(
+                shape_values[index].values
+            )
+            return shape_values[index].values
+
+        feature_importances = np.mean(
+            [
+                np.abs(_get_shape_values_one_sample(shap_values, i))
+                for i in range(len(shap_values))
+            ],
+            axis=0,
+        ).tolist()
+
+        feature_names = train_feature.columns.tolist()
+        feature_importance_dict = dict(zip(feature_names, feature_importances))
+        utils.show_feature_importance(feature_importance_dict)
+        # TODO: save beeswarm and force plot to html or pdf.
+        # Wait for a fix of https://github.com/slundberg/shap/issues/2142.
+
+    def _analyze_metrics_one_dataset(self, dataset_name: str):
         self._show_overall_flag = True
         dataset = self._get_dataset(dataset_name)
         preds = self._get_predictions(dataset)
@@ -94,10 +135,14 @@ class ModelAnalysis:
             + [self.label_to_analyze]
         )
         if dataset_name == "train":
-            return self.data_loader.feature_manager.extract_dataframe(
+            df = self.data_loader.feature_manager.extract_dataframe(
                 features_to_select=all_features,
                 filters=self.data_loader.config.data_loader.train_filters,
             )
+            if self.training_size:
+                return df.sample(n=min(self.training_size, len(df)))
+            return df
+
         if dataset_name == "val":
             return self.data_loader.feature_manager.extract_dataframe(
                 features_to_select=all_features,
